@@ -22,6 +22,9 @@
 #include <QStandardPaths>
 #include <QSignalSpy>
 
+#include <KSharedConfig>
+#include <KConfigGroup>
+
 #include "alternativesmodeltest.h"
 #include <purpose/job.h>
 #include <purpose/alternativesmodel.h>
@@ -55,6 +58,8 @@ void AlternativesModelTest::runJobTest()
     model.setInputData(input);
 
     model.setPluginType(QStringLiteral("Export"));
+    model.setDisabledPlugins({});
+
     Purpose::Configuration* conf = model.configureJob(saveAsRow(&model));
     QVERIFY(!conf->isReady());
     QVERIFY(!conf->createJob());
@@ -91,6 +96,7 @@ void AlternativesModelTest::bigBufferTest()
     };
     model.setInputData(input);
     model.setPluginType(QStringLiteral("Export"));
+    model.setDisabledPlugins({});
 
     Purpose::Configuration* conf = model.configureJob(saveAsRow(&model));
     QVERIFY(conf->isReady());
@@ -111,3 +117,79 @@ void AlternativesModelTest::bigBufferTest()
     QCOMPARE(fi.size(), payload.size());
     QVERIFY(QFile::remove(tempfile));
 }
+
+void AlternativesModelTest::disablePluginTest()
+{
+    const auto listPlugins = [] {
+        QStringList plugins;
+        Purpose::AlternativesModel model;
+        QJsonObject input = QJsonObject {
+            {QStringLiteral("urls"), QJsonArray {QStringLiteral("http://kde.org")} },
+            {QStringLiteral("mimeType"), QStringLiteral("dummy/thing") }
+        };
+        model.setInputData(input);
+        model.setPluginType(QStringLiteral("Export"));
+        model.setDisabledPlugins({});
+
+        for (int i = 0; i < model.rowCount(); ++i) {
+            plugins << model.index(i).data(Purpose::AlternativesModel::PluginIdRole).toString();
+        }
+        return plugins;
+    };
+
+    auto plugins = listPlugins();
+    QVERIFY(plugins.contains(QStringLiteral("saveasplugin")));
+    QVERIFY(plugins.contains(QStringLiteral("emailplugin")));
+
+    QStandardPaths::setTestModeEnabled(true);
+    auto config = KSharedConfig::openConfig(QStringLiteral("purposerc"));
+    auto group = config->group("plugins");
+    group.writeEntry("disabled", QStringList{ QStringLiteral("saveasplugin"), QStringLiteral("emailplugin") });
+
+    plugins = listPlugins();
+    QVERIFY(!plugins.contains(QStringLiteral("saveasplugin")));
+    QVERIFY(!plugins.contains(QStringLiteral("emailplugin")));
+
+    // "cleanup"
+    group.writeEntry("disabled", QStringList());
+}
+
+void AlternativesModelTest::blacklistTest()
+{
+    const auto listPlugins = [](const QStringList &blacklist = QStringList()) {
+        QStringList plugins;
+        Purpose::AlternativesModel model;
+        QJsonObject input = QJsonObject {
+            {QStringLiteral("urls"), QJsonArray {QStringLiteral("http://kde.org")} },
+            {QStringLiteral("mimeType"), QStringLiteral("dummy/thing") }
+        };
+        model.setInputData(input);
+        model.setPluginType(QStringLiteral("Export"));
+        if (!blacklist.isEmpty()) {
+            model.setDisabledPlugins(blacklist);
+        }
+
+        for (int i = 0; i < model.rowCount(); ++i) {
+            plugins << model.index(i).data(Purpose::AlternativesModel::PluginIdRole).toString();
+        }
+        return plugins;
+    };
+
+    auto plugins = listPlugins();
+    QVERIFY(plugins.contains(QStringLiteral("kdeconnectplugin")));
+
+    plugins = listPlugins({QStringLiteral("kdeconnectplugin")});
+    QVERIFY(!plugins.contains(QStringLiteral("kdeconnectplugin")));
+
+    plugins = listPlugins({QStringLiteral("saveasplugin")});
+    QVERIFY(plugins.contains(QStringLiteral("kdeconnectplugin")));
+
+    // Admin settings have precedence
+    QStandardPaths::setTestModeEnabled(true);
+    auto config = KSharedConfig::openConfig(QStringLiteral("purposerc"));
+    auto group = config->group("plugins");
+    group.writeEntry("disabled", QStringList{ QStringLiteral("kdeconnectplugin") });
+
+    plugins = listPlugins({QStringLiteral("saveasplugin")});
+    QVERIFY(!plugins.contains(QStringLiteral("kdeconnectplugin")));
+}
